function [comp_ratio, f1, fw1, err] = complexity_by_wavelets_by_energy(f, params)
% COMPLEXITY_BY_WAVELETS_BY_ENERGY -
% [update to complexy_by_wavelets, 08-04-15]
% compute the complexity ratio of a signal,
% defined as the ratio of (number-of-wavelets-coefficients)/(length-signal)
% needed to get a good reconstruction, defined [Folloiwing Avi Mayo], as
% reconstrcut that captures 90% of the energy of the original signal.
%
%  [comp_rate, f1, fw] = complexity_by_wavelets_by_energy(f,  params) -
% input:
% f - signal, length have to be 2^K
% dist_thr -
% params (optional) -
%   .plot (=0) - plot original and final re-constructed,
%   .energy_thr (=0.99) - needed energy-from-original ratio for reconstruction
%
% output:
% comp_ratio - compression ratio needed for 90% reconstrcution error (higher
% ratio <=> more complex signal)
% xw - signal reconstructed with wavelets
% fw - wavelets vector
% err - reconstruction error ( median(abs(f1 - f)) )

% Default Values for Optional Inputs %
if ~exist('params','var') || isempty(params)
    params = [];
end

params = add_default( params, 'plot',  0 );
params = add_default( params, 'all_plot',  0 );
params = add_default( params, 'energy_thr', 0.99 );

Energy_Thr = params.energy_thr;

% force as col vector, flip at the end back if needed
if size( f, 1 ) < size( f, 2 ),
    f = f';
    flip_back_to_row = 1;
else
    flip_back_to_row = 0;
end

% verify length of signal fit for wavelets
if round(log2(length(f))) ~= log2(length(f)),
    error('length of signal <x> should be 2^N')
end

% wavelets parameters
options.wavelet_vm = 4; % number of vanishing moments
options.wavelet_type = 'daubechies'; % kind of wavelet
Jmin = 3; %  minimum scale

fw_full = perform_wavelet_transform(f,Jmin,+1,options); % wavelets vector

% compute ratio of coeffiecent needed for 90% reconsruction
fe = fw_full .^ 2;
energy_90 = Energy_Thr * sum( fe );

x = [ (1:length(fe))', fe ];
x = sortrows( x, 2 );
x = flipud( x ); % large to small
sum_fe = cumsum( x(:,2) );

% keep only the largest wavelet coeffiecient, needed to get 90% of the original energy
last_ind = find( sum_fe < energy_90, 1, 'last' );
non_relevant_coeff = x( last_ind + 1 : end, 1 );

fw1 = fw_full;
fw1( non_relevant_coeff ) = 0;

% given the relevant coeff, compute reconstructed signal and error
f1 = perform_wavelet_transform(fw1,Jmin,-1,options); %  re-constructed
err = median(abs(f1 - f)); % error
comp_ratio = length(find(fw1)) / length(fw_full);

if params.plot,  
    h(1) = plot(f,'LineWidth',2);
    hold on
    h(2) = plot(f1,'r','LineWidth',2);
    hold off
    axis tight
    legend( h, { 'original signal', 'reconstructed' },'FontSize',12,'Location', 'NorthWest' )
    title( [ remove_underscore( params.lbl ), ': CR = ',num2str(my_round(comp_ratio,3))], 'FontSize',14 )
end

if flip_back_to_row,
    f1 = f1';
    fw1 = fw1';
end
